% PRICE_STATISTICS: Generate price statistics for cellarray of price
% objects using a particular price filter.
%
% usage:
%
% stats = price_statistics(products, method)
%
% where
% 
% products = cellarray of price objects, as generated by extract_and_save.m
% method = 'NS' - Nakamura-Steinsson sale filter
%          'Mid' - Midgrigan/Neilson sales filter
%          'EJR' - Eichenbaum, Jaimovich, Rebelo reference price filter
%          'RAC' - Chahrour Price Filter
%
% stats = an object with a bunch of statstics FOR EACH SERIES


function stats = price_statistics(products, method, param)
ntot = 0;
for jj = 1:length(products)

    %Select current object
    obj = products{jj};
    
    
    %Determine reference price
    if param.skip_per == 1
        price = obj.price';
        exp = obj.exp';
    elseif param.skip_per ==4
        [price, idx_tmp] = week2month(obj.price,param);
        exp = obj.exp(idx_tmp)';
    end
    attr = filter_price(price, method, param);


    %Mark the places where there is both an attr price and a regular price
    %available.
    val_idx  = (~isnan(attr) + ~isnan(price)) == 2;
    np = sum(val_idx);


    %Mark the places where a change is recorded
    val_chg = (val_idx + [0;val_idx(1:end-1)]) == 2;
    nd = sum(val_chg);

    %Play with indexes
    val_idx = find(val_idx);
    val_chg = find(val_chg);
    val_price = price(val_idx);
    val_ref = attr(val_idx);
    val_exp = exp(val_idx);

    %Percent of time at each kind of reference price
    n_ref = sum(val_ref == val_price);
    n_nonref = sum(val_ref~=val_price);
    per_at_ref = n_ref/np;

    %Fraction of Revenue at reference prices
    frac_rev = sum(val_exp(val_ref == val_price))/sum(val_exp);

    %volatility
    stddev = std(log(val_ref));
    d_stddev = std(log_change_in(attr, val_chg));

    %Number of price changes
    up = attr(val_chg) > attr(val_chg-1);   %Increase in attr price
    down = attr(val_chg) < attr(val_chg-1);  %Decrease in attr price
    n_up = sum(up);
    n_down = sum(down);
    n_delta = n_up + n_down;

    %Frequency of Price Change
    per_change = n_delta/(nd);

    %Direction of Price Change
    per_up = n_up/n_delta;
    per_down = n_down/n_delta;

    %Price above/below attractor?
    above = val_price > val_ref;
    below = val_price < val_ref;

    
    %Percent of time above and below
    if n_nonref > 0
        above_ref = sum(above)/(n_nonref);
        below_ref = sum(below)/(n_nonref);
    else
        above_ref = NaN;
        below_ref = NaN;
    end
    
    %Obs Days since sale
    nd_since_sale = NaN*val_idx;

    sale_idx = [0; val_idx(below); max(val_idx)+1];

    for kk = 1:length(sale_idx)-1
        zone = ((val_idx >= sale_idx(kk)) + (val_idx < (sale_idx(kk+1)))) == 2;
        nd_since_sale(zone) = val_idx(zone) - sale_idx(kk);
    end
    temp = NaN*price;
    temp(val_idx) = nd_since_sale;
    nd_since_sale = temp(val_chg);


    %Percent magnitude of price deviations from attractor
    mag_above = sum(log(val_price(above))-log(val_ref(above)))/sum(above);
    mag_below = sum(log(val_price(below))-log(val_ref(below)))/sum(below);


    %Store results in vevtors
    stats.per_at_ref(jj) = per_at_ref;
    stats.frac_rev(jj) = frac_rev;
    stats.stddev(jj) = stddev;
    stats.d_stddev(jj) = d_stddev;
    stats.per_change(jj) = per_change;

    stats.per_up(jj) = per_up;
    stats.per_down(jj) = per_down;

    stats.per_above_ref(jj) =above_ref;
    stats.per_below_ref(jj) =below_ref;

    stats.mag_above(jj) = mag_above;
    stats.mag_below(jj) = mag_below;
    stats.exp_shr(jj) = obj.exp_shr;

    stats.markov(:,:,jj) = markov_est(price, attr, val_chg);
    stats.up{jj} = up;
    stats.down{jj} = down;
    stats.nd_since{jj} = nd_since_sale;
    
    ntot = ntot+np;
end
stats.ntot = ntot;
stats.nobs = length(stats.per_at_ref);


%Subfunction for convenience
function out = log_change_in(data, val_chg)

out = log(data(val_chg)) - log(data(val_chg-1));

